@@ -1,6 +1,6 @@
 // info.h
 
-// Copyright (C) 2012-2014 by Werner Lemberg.
+// Copyright (C) 2012-2015 by Werner Lemberg.
 //
 // This file is part of the ttfautohint library, and may only be used,
 // modified, and distributed under the terms given in `COPYING'.  By
@@ -15,15 +15,22 @@
 #define __INFO_H__
 
 #include <ttfautohint.h>
+#include <stdbool.h> // for llrb.h
+#include <llrb.h>
 
 extern "C" {
 
 typedef struct Info_Data_
 {
-  unsigned char* data;
-  unsigned char* data_wide;
-  unsigned short data_len;
-  unsigned short data_wide_len;
+  bool no_info;
+  bool detailed_info;
+  unsigned char* info_string;
+  unsigned char* info_string_wide;
+  unsigned short info_string_len;
+  unsigned short info_string_wide_len;
+
+  const char* family_suffix;
+  void* family_data_head; // a red-black tree
 
   int hinting_range_min;
   int hinting_range_max;
@@ -50,10 +57,13 @@ typedef struct Info_Data_
 } Info_Data;
 
 
+char*
+check_family_suffix(const char* s);
+
 int
 build_version_string(Info_Data* idata);
 
-TA_Error
+int
 info(unsigned short platform_id,
      unsigned short encoding_id,
      unsigned short language_id,
@@ -62,6 +72,9 @@ info(unsigned short platform_id,
      unsigned char** str,
      void* user);
 
+int
+info_post(void* user);
+
 } // extern "C"
 
 #endif // __INFO_H__
