@@ -1,6 +1,6 @@
 // main.cpp
 
-// Copyright (C) 2011-2014 by Werner Lemberg.
+// Copyright (C) 2011-2015 by Werner Lemberg.
 //
 // This file is part of the ttfautohint library, and may only be used,
 // modified, and distributed under the terms given in `COPYING'.  By
@@ -62,25 +62,39 @@
 using namespace std;
 
 
-// the available script tags and its descriptions are directly extracted
-// from `ttfautohint-scripts.h'
-typedef struct Script_Names_
+typedef struct Tag_Names_
 {
   const char* tag;
   const char* description;
-} Script_Names;
+} Tag_Names;
 
+
+// the available script tags and its descriptions are directly extracted
+// from `ttfautohint-scripts.h'
 #undef SCRIPT
 #define SCRIPT(s, S, d, h, sc1, sc2, sc3) \
           {#s, d},
 
-const Script_Names script_names[] =
+const Tag_Names script_names[] =
 {
 #include <ttfautohint-scripts.h>
   {NULL, NULL}
 };
 
 
+// the available feature tags and its descriptions are directly extracted
+// from `ttfautohint-coverages.h'
+#undef COVERAGE
+#define COVERAGE(n, N, d, t, t1, t2, t3, t4) \
+          {#t, d},
+
+const Tag_Names feature_names[] =
+{
+#include <ttfautohint-coverages.h>
+  {NULL, NULL}
+};
+
+
 #ifndef BUILD_GUI
 extern "C" {
 
@@ -92,7 +106,7 @@ typedef struct Progress_Data_
 } Progress_Data;
 
 
-int
+static int
 progress(long curr_idx,
          long num_glyphs,
          long curr_sfnt,
@@ -139,7 +153,7 @@ typedef struct Error_Data_
 } Error_Data;
 
 
-void
+static void
 err(TA_Error error,
     const char* error_string,
     unsigned int errlinenum,
@@ -189,7 +203,7 @@ err(TA_Error error,
   {
     if (error < 0x100)
       fprintf(stderr, "An error with code 0x%02x occurred"
-                        " while autohinting fonts",
+                        " while autohinting fonts\n",
                       error);
     else if (error >= 0x100 && error < 0x200)
     {
@@ -275,6 +289,8 @@ show_help(bool
 "  -d, --dehint               remove all hints\n"
 "  -D, --default-script=S     set default OpenType script (default: latn)\n"
 "  -f, --fallback-script=S    set fallback script (default: none)\n"
+"  -F, --family-suffix=S      append suffix to the family name string(s)\n"
+"                             in the `name' table\n"
 "  -G, --hinting-limit=N      switch off hinting above this PPEM value\n"
 "                             (default: %d); value 0 means no limit\n"
 "  -h, --help                 display this help and exit\n"
@@ -285,6 +301,8 @@ show_help(bool
 "      --help-all             show Qt and X11 specific options also\n"
 #endif
 "  -i, --ignore-restrictions  override font license restrictions\n"
+"  -I, --detailed-info        add detailed ttfautohint info\n"
+"                             to the version string(s) in the `name' table\n"
 "  -l, --hinting-range-min=N  the minimum PPEM value for hint sets\n"
 "                             (default: %d)\n"
 #ifndef BUILD_GUI
@@ -393,7 +411,7 @@ show_help(bool
 "character codes outside of known script ranges.  Option -D sets the\n"
 "default script for handling OpenType features.  Possible values are\n"
 "\n");
-  const Script_Names* sn = script_names;
+  const Tag_Names* sn = script_names;
   for(;;)
   {
     fprintf(handle, "  %s (%s)",
@@ -412,22 +430,49 @@ show_help(bool
 "\n"
 "A control instructions file contains entries of the form\n"
 "\n"
-"  [<subfont idx>] <glyph id> l|r <points> [(<left offset>,<right offset>)]\n"
-"or\n"
-"  [<subfont idx>] <glyph id> n <points>\n"
-"or\n"
-"  [<subfont idx>] <glyph id> p <points> [x <shift>] [y <shift>] @ <ppems>\n"
+"  [<font idx>] <script> <feature> @ <glyph ids>\n"
+"\n"
+"  [<font idx>] <glyph id> l|r <points> [(<left offset>,<right offset>)]\n"
+"\n"
+"  [<font idx>] <glyph id> n <points>\n"
+"\n"
+"  [<font idx>] <glyph id> t|p <points> [x <shift>] [y <shift>] @ <ppems>\n"
+"\n"
+"<font idx> is the current subfont, <glyph id> is a glyph name or index,\n"
+"<glyph ids> is a set of <glyph id>s, <shift> is a real number in px,\n"
+"<points> and <ppems> are integer ranges as with option `-X'.\n"
+"\n"
+"<script> and <feature> are four-letter tags that define a style\n"
+"the <glyph ids> are assigned to; possible values for <script> are the same\n"
+"as with option -D, possible values for <feature> are\n"
+"\n");
+  const Tag_Names* fn = feature_names;
+  for(;;)
+  {
+    fprintf(handle, "  %s (%s)",
+            fn->tag, fn->description);
+    fn++;
+    if (fn->tag)
+      fprintf(handle, ",\n");
+    else
+    {
+      fprintf(handle, ".\n");
+      break;
+    }
+  }
+  fprintf(handle,
 "\n"
 "`l' (`r') creates one-point segments with direction left (right).\n"
 "<left offset> and <right offset> specify offsets (in font units)\n"
 "relative to the corresponding points to give the segments a length.\n"
 "`n' removes points from horizontal segments, making them `weak' points.\n"
-"`p' applies delta exceptions to the specified points.\n"
+"`t' (`p') applies delta exceptions to the given points before (after) IUP.\n"
 "\n"
-"<glyph id> is a glyph name or index, <shift> is a real number in px,\n"
-"<points> and <ppems> are integer ranges as with option `-X'.\n"
 "`#' starts a line comment, which gets ignored.\n"
 "Empty lines are ignored, too.\n"
+"\n"
+"Key letters `l', `r', `n', `p', `t', `x', and `y' have the verbose aliases\n"
+"`left', `right', `nodir', `point', `touch', `xshift', and `yshift'.\n"
 #endif
 "\n"
 #ifdef BUILD_GUI
@@ -437,6 +482,7 @@ show_help(bool
 #endif
 "\n"
 "Report bugs to: freetype-devel@nongnu.org\n"
+"\n"
 "ttfautohint home page: <http://www.freetype.org/ttfautohint>\n");
 
   if (is_error)
@@ -455,7 +501,7 @@ show_version()
 #else
 "ttfautohint " VERSION "\n"
 #endif
-"Copyright (C) 2011-2014 Werner Lemberg <wl@gnu.org>.\n"
+"Copyright (C) 2011-2015 Werner Lemberg <wl@gnu.org>.\n"
 "License: FreeType License (FTL) or GNU GPLv2.\n"
 "This is free software: you are free to change and redistribute it.\n"
 "There is NO WARRANTY, to the extent permitted by law.\n");
@@ -490,6 +536,7 @@ main(int argc,
   bool adjust_subglyphs = false;
   bool hint_composites = false;
   bool no_info = false;
+  bool detailed_info = false;
   bool TTFA_info = false;
   bool symbol = false;
 
@@ -499,6 +546,8 @@ main(int argc,
   bool have_fallback_script = false;
   const char* x_height_snapping_exceptions_string = NULL;
   bool have_x_height_snapping_exceptions_string = false;
+  const char* family_suffix = NULL;
+  bool have_family_suffix = false;
 
   bool dehint = false;
 
@@ -508,6 +557,7 @@ main(int argc,
   TA_Progress_Func progress_func = NULL;
   TA_Error_Func err_func = err;
   TA_Info_Func info_func = info;
+  TA_Info_Post_Func info_post_func = info_post;
 
   const char* control_name = NULL;
 #endif
@@ -551,8 +601,10 @@ main(int argc,
 #endif
       {"default-script", required_argument, NULL, 'D'},
       {"dehint", no_argument, NULL, 'd'},
+      {"detailed-info", no_argument, NULL, 'I'},
       {"fallback-script", required_argument, NULL, 'f'},
       {"fallback-stem-width", required_argument, NULL, 'H'},
+      {"family-suffix", required_argument, NULL, 'F'},
       {"hinting-limit", required_argument, NULL, 'G'},
       {"hinting-range-max", required_argument, NULL, 'r'},
       {"hinting-range-min", required_argument, NULL, 'l'},
@@ -600,9 +652,9 @@ main(int argc,
     int option_index;
     int c = getopt_long_only(argc, argv,
 #ifdef BUILD_GUI
-                             "cdD:f:G:hH:il:npr:stVvw:Wx:X:",
+                             "cdD:f:F:G:hH:iIl:npr:stVvw:Wx:X:",
 #else
-                             "cdD:f:G:hH:il:m:npr:stVvw:Wx:X:",
+                             "cdD:f:F:G:hH:iIl:m:npr:stVvw:Wx:X:",
 #endif
                              long_options, &option_index);
     if (c == -1)
@@ -628,6 +680,11 @@ main(int argc,
       have_fallback_script = true;
       break;
 
+    case 'F':
+      family_suffix = optarg;
+      have_family_suffix = true;
+      break;
+
     case 'G':
       hinting_limit = atoi(optarg);
       have_hinting_limit = true;
@@ -648,6 +705,11 @@ main(int argc,
       ignore_restrictions = true;
       break;
 
+    case 'I':
+      detailed_info = true;
+      no_info = false;
+      break;
+
     case 'l':
       hinting_range_min = atoi(optarg);
       have_hinting_range_min = true;
@@ -661,6 +723,7 @@ main(int argc,
 
     case 'n':
       no_info = true;
+      detailed_info = false;
       break;
 
     case 'p':
@@ -773,6 +836,8 @@ main(int argc,
     x_height_snapping_exceptions_string = "";
   if (!have_fallback_stem_width)
     fallback_stem_width = 0; /* redundant, but avoids a compiler warning */
+  if (!have_family_suffix)
+    family_suffix = "";
 
 #ifndef BUILD_GUI
 
@@ -813,7 +878,7 @@ main(int argc,
 
   if (have_default_script)
   {
-    const Script_Names* sn;
+    const Tag_Names* sn;
 
     for (sn = script_names; sn->tag; sn++)
       if (!strcmp(default_script, sn->tag))
@@ -827,7 +892,7 @@ main(int argc,
 
   if (have_fallback_script)
   {
-    const Script_Names* sn;
+    const Tag_Names* sn;
 
     for (sn = script_names; sn->tag; sn++)
       if (!strcmp(fallback_script, sn->tag))
@@ -846,6 +911,17 @@ main(int argc,
             "Warning: Setting a fallback stem width for a symbol font\n"
             "         without setting a fallback script has no effect\n");
 
+  if (const char* pos = check_family_suffix(family_suffix))
+  {
+    fprintf(stderr,
+            "Invalid character in family suffix:\n"
+            "  %s\n"
+            "  %*s\n",
+            family_suffix,
+            int(pos - family_suffix + 1), "^");
+    exit(EXIT_FAILURE);
+  }
+
   int num_args = argc - optind;
 
   if (num_args > 2)
@@ -920,43 +996,48 @@ main(int argc,
   Error_Data error_data = {control_name};
   Info_Data info_data;
 
-  if (no_info)
-    info_func = NULL;
-  else
+  if (!*family_suffix)
+    info_post_func = NULL;
+
+  info_data.no_info = no_info;
+  info_data.detailed_info = detailed_info;
+  info_data.info_string = NULL; // must be deallocated after use
+  info_data.info_string_wide = NULL; // must be deallocated after use
+  info_data.info_string_len = 0;
+  info_data.info_string_wide_len = 0;
+
+  info_data.control_name = control_name;
+
+  info_data.hinting_range_min = hinting_range_min;
+  info_data.hinting_range_max = hinting_range_max;
+  info_data.hinting_limit = hinting_limit;
+
+  info_data.gray_strong_stem_width = gray_strong_stem_width;
+  info_data.gdi_cleartype_strong_stem_width = gdi_cleartype_strong_stem_width;
+  info_data.dw_cleartype_strong_stem_width = dw_cleartype_strong_stem_width;
+
+  info_data.windows_compatibility = windows_compatibility;
+  info_data.adjust_subglyphs = adjust_subglyphs;
+  info_data.hint_composites = hint_composites;
+  info_data.increase_x_height = increase_x_height;
+  info_data.x_height_snapping_exceptions_string = x_height_snapping_exceptions_string;
+  info_data.family_suffix = family_suffix;
+  info_data.family_data_head = NULL;
+  info_data.fallback_stem_width = fallback_stem_width;
+  info_data.symbol = symbol;
+  info_data.TTFA_info = TTFA_info;
+
+  strncpy(info_data.default_script,
+          default_script,
+          sizeof (info_data.default_script));
+  strncpy(info_data.fallback_script,
+          fallback_script,
+          sizeof (info_data.fallback_script));
+
+  info_data.dehint = dehint;
+
+  if (!no_info)
   {
-    info_data.data = NULL; // must be deallocated after use
-    info_data.data_wide = NULL; // must be deallocated after use
-    info_data.data_len = 0;
-    info_data.data_wide_len = 0;
-
-    info_data.control_name = control_name;
-
-    info_data.hinting_range_min = hinting_range_min;
-    info_data.hinting_range_max = hinting_range_max;
-    info_data.hinting_limit = hinting_limit;
-
-    info_data.gray_strong_stem_width = gray_strong_stem_width;
-    info_data.gdi_cleartype_strong_stem_width = gdi_cleartype_strong_stem_width;
-    info_data.dw_cleartype_strong_stem_width = dw_cleartype_strong_stem_width;
-
-    info_data.windows_compatibility = windows_compatibility;
-    info_data.adjust_subglyphs = adjust_subglyphs;
-    info_data.hint_composites = hint_composites;
-    info_data.increase_x_height = increase_x_height;
-    info_data.x_height_snapping_exceptions_string = x_height_snapping_exceptions_string;
-    info_data.fallback_stem_width = fallback_stem_width;
-    info_data.symbol = symbol;
-    info_data.TTFA_info = TTFA_info;
-
-    strncpy(info_data.default_script,
-            default_script,
-            sizeof (info_data.default_script));
-    strncpy(info_data.fallback_script,
-            fallback_script,
-            sizeof (info_data.fallback_script));
-
-    info_data.dehint = dehint;
-
     int ret = build_version_string(&info_data);
     if (ret == 1)
       fprintf(stderr, "Warning: Can't allocate memory"
@@ -978,7 +1059,7 @@ main(int argc,
                  "dw-cleartype-strong-stem-width,"
                  "progress-callback, progress-callback-data,"
                  "error-callback, error-callback-data,"
-                 "info-callback, info-callback-data,"
+                 "info-callback, info-post-callback, info-callback-data,"
                  "ignore-restrictions, windows-compatibility,"
                  "adjust-subglyphs, hint-composites,"
                  "increase-x-height, x-height-snapping-exceptions,"
@@ -990,7 +1071,7 @@ main(int argc,
                  dw_cleartype_strong_stem_width,
                  progress_func, &progress_data,
                  err_func, &error_data,
-                 info_func, &info_data,
+                 info_func, info_post_func, &info_data,
                  ignore_restrictions, windows_compatibility,
                  adjust_subglyphs, hint_composites,
                  increase_x_height, x_height_snapping_exceptions_string,
@@ -999,8 +1080,8 @@ main(int argc,
 
   if (!no_info)
   {
-    free(info_data.data);
-    free(info_data.data_wide);
+    free(info_data.info_string);
+    free(info_data.info_string_wide);
   }
 
   if (in != stdin)
@@ -1016,7 +1097,7 @@ main(int argc,
 
 #else // BUILD_GUI
 
-  int new_argc = new_arg_string.size();
+  int new_argc = (int)new_arg_string.size();
   char** new_argv = new char*[new_argc];
 
   // construct new argc and argv variables from collected data
@@ -1058,7 +1139,8 @@ main(int argc,
                    dw_cleartype_strong_stem_width, increase_x_height,
                    x_height_snapping_exceptions_string, fallback_stem_width,
                    ignore_restrictions, windows_compatibility, adjust_subglyphs,
-                   hint_composites, no_info, default_script, fallback_script,
+                   hint_composites, no_info, detailed_info,
+                   default_script, fallback_script, family_suffix,
                    symbol, dehint, TTFA_info);
 
     dummy.move(-50000, -50000);
@@ -1067,7 +1149,7 @@ main(int argc,
     // if the vertical size of our window is too large,
     // select a horizontal layout
     QRect screen(QApplication::desktop()->availableGeometry());
-    if (dummy.frameGeometry().height() > screen.width())
+    if (dummy.frameGeometry().height() > screen.height())
       alternative_layout = true;
   }
 
@@ -1077,7 +1159,8 @@ main(int argc,
                dw_cleartype_strong_stem_width, increase_x_height,
                x_height_snapping_exceptions_string, fallback_stem_width,
                ignore_restrictions, windows_compatibility, adjust_subglyphs,
-               hint_composites, no_info, default_script, fallback_script,
+               hint_composites, no_info, detailed_info,
+               default_script, fallback_script, family_suffix,
                symbol, dehint, TTFA_info);
   gui.show();
 
