@@ -180,9 +180,13 @@ FT_Byte ttfautohint_glyph_bytecode[7] =
 };
 
 
-/* if we have y delta exceptions before IUP_y, this code gets inserted */
+/* a buffer holding additional instructions to be inserted */
+/* at the beginning of the bytecode */
+FT_Byte ins_extra_buf[256];
 
-FT_Byte ins_extra_buf[4] =
+
+/* if we have y delta exceptions before IUP_y, this code gets inserted */
+FT_Byte ins_extra_delta_exceptions[4] =
 {
 
   /* tell bci_{scale,scale_composite,hint}_glyph to not call IUP_y */
@@ -194,6 +198,19 @@ FT_Byte ins_extra_buf[4] =
 };
 
 
+/* if we have a non-base glyph, this code gets inserted */
+FT_Byte ins_extra_ignore_std_width[4] =
+{
+
+  /* tell bci_{smooth,strong}_stem_width to ignore std_width */
+  PUSHB_2,
+    cvtl_ignore_std_width,
+    100,
+  WCVTP,
+
+};
+
+
 /*
  * convert array `args' into a sequence of NPUSHB, NPUSHW, PUSHB_X, and
  * PUSHW_X instructions to be stored in `bufp' (the latter two instructions
@@ -270,7 +287,7 @@ TA_build_push(FT_Byte* bufp,
  * if possible
  */
 
-FT_Byte*
+static FT_Byte*
 TA_optimize_push(FT_Byte* buf,
                  FT_Byte** pos)
 {
@@ -316,12 +333,12 @@ TA_optimize_push(FT_Byte* buf,
   {
     /* reduce three NPUSHB to two */
     new_size1 = 0xFF;
-    new_size2 = sum - 0xFF;
+    new_size2 = (FT_Byte)(sum - 0xFF);
   }
   else
   {
     /* reduce two or three NPUSHB to one */
-    new_size1 = sum;
+    new_size1 = (FT_Byte)sum;
     new_size2 = 0;
   }
 
@@ -443,7 +460,7 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
 
 
   seg_limit = segments + axis->num_segments;
-  num_segments = axis->num_segments;
+  num_segments = (FT_UShort)axis->num_segments;
 
   /* to pack the data in the bytecode more tightly, */
   /* we store up to the first nine segments in nibbles if possible, */
@@ -452,8 +469,8 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
   num_packed_segments = 0;
   for (seg = segments; seg < seg_limit; seg++)
   {
-    FT_UInt first = seg->first - points;
-    FT_UInt last = seg->last - points;
+    FT_UInt first = (FT_UInt)(seg->first - points);
+    FT_UInt last = (FT_UInt)(seg->last - points);
 
 
     first = TA_adjust_point_index(recorder, first);
@@ -503,8 +520,8 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
   base = 0;
   for (seg = segments; seg < segments + num_packed_segments; seg++)
   {
-    FT_UInt first = seg->first - points;
-    FT_UInt last = seg->last - points;
+    FT_UInt first = (FT_UInt)(seg->first - points);
+    FT_UInt last = (FT_UInt)(seg->last - points);
     FT_UInt low_nibble;
     FT_UInt high_nibble;
 
@@ -522,8 +539,8 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
 
   for (seg = segments + num_packed_segments; seg < seg_limit; seg++)
   {
-    FT_UInt first = seg->first - points;
-    FT_UInt last = seg->last - points;
+    FT_UInt first = (FT_UInt)(seg->first - points);
+    FT_UInt last = (FT_UInt)(seg->last - points);
 
 
     *(arg--) = TA_adjust_point_index(recorder, first);
@@ -562,8 +579,8 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
   /* so that edges can easily link to them */
   for (seg = segments; seg < seg_limit; seg++)
   {
-    FT_UInt first = seg->first - points;
-    FT_UInt last = seg->last - points;
+    FT_UInt first = (FT_UInt)(seg->first - points);
+    FT_UInt last = (FT_UInt)(seg->last - points);
 
 
     if (first > last)
@@ -602,8 +619,9 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
 
   /* both this function and `TA_emit_hints_record' */
   /* push data onto the stack */
-  num_stack_elements = ADDITIONAL_STACK_ELEMENTS
-                       + recorder->num_stack_elements + num_args;
+  num_stack_elements = (FT_UShort)(ADDITIONAL_STACK_ELEMENTS
+                                   + recorder->num_stack_elements
+                                   + num_args);
   if (num_stack_elements > sfnt->max_stack_elements)
     sfnt->max_stack_elements = num_stack_elements;
 
@@ -616,7 +634,7 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
 static void
 build_delta_exception(const Ctrl* ctrl,
                       FT_UInt** delta_args,
-                      int* num_delta_args)
+                      unsigned int* num_delta_args)
 {
   int offset;
   int ppem;
@@ -668,18 +686,18 @@ build_delta_exception(const Ctrl* ctrl,
   if (ctrl->x_shift)
   {
     *(delta_args[offset] + num_delta_args[offset]++) =
-      (ppem << 4) + x_shift;
+      (FT_UInt)((ppem << 4) + x_shift);
     *(delta_args[offset] + num_delta_args[offset]++) =
-      ctrl->point_idx;
+      (FT_UInt)ctrl->point_idx;
   }
 
   if (ctrl->y_shift)
   {
     offset += 3;
     *(delta_args[offset] + num_delta_args[offset]++) =
-      (ppem << 4) + y_shift;
+      (FT_UInt)((ppem << 4) + y_shift);
     *(delta_args[offset] + num_delta_args[offset]++) =
-      ctrl->point_idx;
+      (FT_UInt)ctrl->point_idx;
   }
 }
 
@@ -696,17 +714,17 @@ TA_sfnt_build_delta_exceptions(SFNT* sfnt,
 
   FT_Face face = font->loader->face;
 
-  int num_points;
+  unsigned int num_points;
   int i;
 
-  FT_UShort num_before_IUP_stack_elements;
-  FT_UShort num_after_IUP_stack_elements;
+  FT_UShort num_before_IUP_stack_elements = 0;
+  FT_UShort num_after_IUP_stack_elements = 0;
 
   /* DELTAP[1-3] stacks for both x and y directions */
   FT_UInt* delta_before_IUP_args[6] = {NULL, NULL, NULL, NULL, NULL, NULL};
   FT_UInt* delta_after_IUP_args[6] = {NULL, NULL, NULL, NULL, NULL, NULL};
-  int num_delta_before_IUP_args[6] = {0, 0, 0, 0, 0, 0};
-  int num_delta_after_IUP_args[6] = {0, 0, 0, 0, 0, 0};
+  unsigned int num_delta_before_IUP_args[6] = {0, 0, 0, 0, 0, 0};
+  unsigned int num_delta_after_IUP_args[6] = {0, 0, 0, 0, 0, 0};
   FT_UInt* args = NULL;
 
   FT_Bool need_before_IUP_words = 0;
@@ -717,7 +735,7 @@ TA_sfnt_build_delta_exceptions(SFNT* sfnt,
   FT_Bool allocated_after_IUP = 0;
 
 
-  num_points = font->loader->gloader->base.outline.n_points;
+  num_points = (unsigned int)font->loader->gloader->base.outline.n_points;
 
   /* loop over all fitting control instructions */
   for (;;)
@@ -814,7 +832,7 @@ TA_sfnt_build_delta_exceptions(SFNT* sfnt,
   {
     if (num_delta_before_IUP_args[i])
     {
-      int n = num_delta_before_IUP_args[i] >> 1;
+      unsigned int n = num_delta_before_IUP_args[i] >> 1;
 
 
       if (n > 255)
@@ -834,7 +852,7 @@ TA_sfnt_build_delta_exceptions(SFNT* sfnt,
   {
     if (num_delta_after_IUP_args[i])
     {
-      int n = num_delta_after_IUP_args[i] >> 1;
+      unsigned int n = num_delta_after_IUP_args[i] >> 1;
 
 
       if (n > 255)
@@ -877,7 +895,7 @@ TA_sfnt_build_delta_exceptions(SFNT* sfnt,
       num_args = num_args_new;
     }
 
-    num_before_IUP_stack_elements = num_args;
+    num_before_IUP_stack_elements = (FT_UShort)num_args;
 
     bufp = TA_build_push(bufp, args, num_args, need_before_IUP_words, 1);
   }
@@ -888,7 +906,7 @@ TA_sfnt_build_delta_exceptions(SFNT* sfnt,
     /* stack elements are bytes, but counts need words */
     for (i = 0; i < 6; i++)
     {
-      int num_delta_arg;
+      FT_UInt num_delta_arg;
 
 
       if (!num_delta_before_IUP_args[i])
@@ -978,7 +996,7 @@ TA_sfnt_build_delta_exceptions(SFNT* sfnt,
       num_args = num_args_new;
     }
 
-    num_after_IUP_stack_elements = num_args;
+    num_after_IUP_stack_elements = (FT_UShort)num_args;
 
     bufp = TA_build_push(bufp, args, num_args, need_after_IUP_words, 1);
   }
@@ -989,7 +1007,7 @@ TA_sfnt_build_delta_exceptions(SFNT* sfnt,
     /* stack elements are bytes, but counts need words */
     for (i = 0; i < 6; i++)
     {
-      int num_delta_arg;
+      FT_UInt num_delta_arg;
 
 
       if (!num_delta_after_IUP_args[i])
@@ -1040,8 +1058,11 @@ TA_sfnt_build_delta_exceptions(SFNT* sfnt,
       || num_delta_before_IUP_args[3])
   {
     /* set `cvtl_do_iup_y' to zero at the beginning of the bytecode */
-    /* by activating `ins_extra_buf' */
-    glyph->ins_extra_len = sizeof (ins_extra_buf) / sizeof (FT_Byte);
+    /* by activating `ins_extra_delta_exceptions' */
+    memcpy(ins_extra_buf,
+           ins_extra_delta_exceptions,
+           sizeof (ins_extra_delta_exceptions));
+    glyph->ins_extra_len += sizeof (ins_extra_delta_exceptions);
 
     /* reset `cvtl_do_iup_y' for next glyph */
     BCI(PUSHB_2);
@@ -1075,15 +1096,15 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
   FONT* font = recorder->font;
   FT_GlyphSlot glyph = sfnt->face->glyph;
   FT_Vector* points = glyph->outline.points;
-  FT_Int num_contours = glyph->outline.n_contours;
+  FT_UInt num_contours = (FT_UInt)glyph->outline.n_contours;
 
   FT_UInt* args;
   FT_UInt* arg;
   FT_UInt num_args;
 
   FT_Bool need_words = 0;
-  FT_Int p, q;
-  FT_Int start, end;
+  FT_UInt p, q;
+  FT_UInt start, end;
   FT_UShort num_storage;
   FT_UShort num_stack_elements;
 
@@ -1113,11 +1134,11 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
 
   for (p = 0; p < num_contours; p++)
   {
-    FT_Int max = start;
-    FT_Int min = start;
+    FT_UInt max = start;
+    FT_UInt min = start;
 
 
-    end = glyph->outline.contours[p];
+    end = (FT_UInt)glyph->outline.contours[p];
 
     for (q = start; q <= end; q++)
     {
@@ -1155,7 +1176,7 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
   if (num_storage > sfnt->max_storage)
     sfnt->max_storage = num_storage;
 
-  num_stack_elements = ADDITIONAL_STACK_ELEMENTS + num_args;
+  num_stack_elements = (FT_UShort)(ADDITIONAL_STACK_ELEMENTS + num_args);
   if (num_stack_elements > sfnt->max_stack_elements)
     sfnt->max_stack_elements = num_stack_elements;
 
@@ -1192,7 +1213,7 @@ TA_font_build_subglyph_shifter(FONT* font,
 
 
     /* load subglyph to get the number of contours */
-    error = FT_Load_Glyph(face, subglyph->index, FT_LOAD_NO_SCALE);
+    error = FT_Load_Glyph(face, (FT_UInt)subglyph->index, FT_LOAD_NO_SCALE);
     if (error)
       return NULL;
     num_contours = glyph->outline.n_contours;
@@ -1721,7 +1742,7 @@ TA_emit_hints_record(Recorder* recorder,
 
   /* collect stack depth data */
   if (num_arguments > recorder->num_stack_elements)
-    recorder->num_stack_elements = num_arguments;
+    recorder->num_stack_elements = (FT_UShort)num_arguments;
 
   return bufp;
 }
@@ -1800,7 +1821,7 @@ TA_hints_recorder_handle_segments(FT_Byte* bufp,
   FT_UShort* wrap;
 
 
-  seg_idx = edge->first - segments;
+  seg_idx = (FT_UShort)(edge->first - segments);
 
   /* we store everything as 16bit numbers */
   *(bufp++) = HIGH(seg_idx);
@@ -1843,7 +1864,7 @@ TA_hints_recorder_handle_segments(FT_Byte* bufp,
   seg = edge->first->edge_next;
   while (seg != edge->first)
   {
-    seg_idx = seg - segments;
+    seg_idx = (FT_UShort)(seg - segments);
 
     *(bufp++) = HIGH(seg_idx);
     *(bufp++) = LOW(seg_idx);
@@ -1908,7 +1929,7 @@ TA_hints_recorder(TA_Action action,
       before_node = (Node1*)malloc(sizeof (Node1));
       if (!before_node)
         return;
-      before_node->point = point - points;
+      before_node->point = (FT_UShort)(point - points);
 
       LLRB_INSERT(ip_before_points,
                   &recorder->ip_before_points_head,
@@ -1925,7 +1946,7 @@ TA_hints_recorder(TA_Action action,
       after_node = (Node1*)malloc(sizeof (Node1));
       if (!after_node)
         return;
-      after_node->point = point - points;
+      after_node->point = (FT_UShort)(point - points);
 
       LLRB_INSERT(ip_after_points,
                   &recorder->ip_after_points_head,
@@ -1943,8 +1964,8 @@ TA_hints_recorder(TA_Action action,
       on_node = (Node2*)malloc(sizeof (Node2));
       if (!on_node)
         return;
-      on_node->edge = edge - edges;
-      on_node->point = point - points;
+      on_node->edge = (FT_UShort)(edge - edges);
+      on_node->point = (FT_UShort)(point - points);
 
       LLRB_INSERT(ip_on_points,
                   &recorder->ip_on_points_head,
@@ -1963,9 +1984,9 @@ TA_hints_recorder(TA_Action action,
       between_node = (Node3*)malloc(sizeof (Node3));
       if (!between_node)
         return;
-      between_node->before_edge = before - edges;
-      between_node->after_edge = after - edges;
-      between_node->point = point - points;
+      between_node->before_edge = (FT_UShort)(before - edges);
+      between_node->after_edge = (FT_UShort)(after - edges);
+      between_node->point = (FT_UShort)(point - points);
 
       LLRB_INSERT(ip_between_points,
                   &recorder->ip_between_points_head,
@@ -2268,7 +2289,7 @@ TA_init_recorder(Recorder* recorder,
   recorder->sfnt = sfnt;
   recorder->font = font;
   recorder->glyph = glyph;
-  recorder->num_segments = axis->num_segments;
+  recorder->num_segments = (FT_UShort)axis->num_segments;
 
   LLRB_INIT(&recorder->ip_before_points_head);
   LLRB_INIT(&recorder->ip_after_points_head);
@@ -2294,7 +2315,7 @@ TA_init_recorder(Recorder* recorder,
   wrap_around_segment = recorder->wrap_around_segments;
   for (seg = segments; seg < seg_limit; seg++)
     if (seg->first > seg->last)
-      *(wrap_around_segment++) = seg - segments;
+      *(wrap_around_segment++) = (FT_UShort)(seg - segments);
 
   /* get number of strong points */
   for (point = points; point < point_limit; point++)
@@ -2430,6 +2451,10 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
   /* `idx' is never negative */
   GLYPH* glyph = &data->glyphs[idx];
 
+  TA_FaceGlobals globals = (TA_FaceGlobals)sfnt->face->autohint.data;
+  FT_UShort* gstyles = globals->glyph_styles;
+  FT_Bool use_gstyle_data = 1;
+
   TA_GlyphHints hints;
 
   FT_UInt num_action_hints_records = 0;
@@ -2523,8 +2548,9 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
    * In summary, this is approx. 3*16 * 2*2 * 2 = 400 bytes per point,
    * adding some bytes for the necessary overhead.
    */
-  ins_len = hints->num_points
-            * (1000 + ((font->control_data_head != NULL) ? 400 : 0));
+  ins_len = (FT_UInt)(hints->num_points
+                      * (1000
+                         + ((font->control_data_head != NULL) ? 400 : 0)));
   ins_buf = (FT_Byte*)malloc(ins_len);
   if (!ins_buf)
     return FT_Err_Out_Of_Memory;
@@ -2539,6 +2565,8 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
       goto Err;
     }
 
+    use_gstyle_data = 0;
+
     goto Done1;
   }
 
@@ -2558,6 +2586,8 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
       goto Err;
     }
 
+    use_gstyle_data = 0;
+
     goto Done1;
   }
 
@@ -2575,7 +2605,7 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
     char buf[256];
 
 
-    (void)FT_Get_Glyph_Name(face, idx, buf, 256);
+    (void)FT_Get_Glyph_Name(face, (FT_UInt)idx, buf, 256);
 
     num_chars = fprintf(stderr, "glyph %ld", idx);
     if (*buf)
@@ -2624,7 +2654,7 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
     /* calling `ta_loader_load_glyph' uses the */
     /* `TA_hints_recorder' function as a callback, */
     /* modifying `hints_record' */
-    error = ta_loader_load_glyph(font, face, idx, load_flags);
+    error = ta_loader_load_glyph(font, face, (FT_UInt)idx, load_flags);
     if (error)
       goto Err;
 
@@ -2721,6 +2751,8 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
       goto Err;
     }
 
+    use_gstyle_data = 0;
+
     goto Done;
   }
 
@@ -2777,10 +2809,28 @@ Done1:
     }
   }
 
-  ins_len = bufp - ins_buf;
+  /* we need to insert a few extra bytecode instructions */
+  /* for non-base glyphs */
+  if (use_gstyle_data && (gstyles[idx] & TA_NONBASE))
+  {
+    /* set `cvtl_ignore_std_width' to 100 at the beginning of the bytecode */
+    /* by activating `ins_extra_ignore_std_width' */
+    memcpy(ins_extra_buf + glyph->ins_extra_len,
+           ins_extra_ignore_std_width,
+           sizeof (ins_extra_ignore_std_width));
+    glyph->ins_extra_len += sizeof (ins_extra_ignore_std_width);
+
+    /* reset `cvtl_ignore_std_width' for next glyph */
+    BCI(PUSHB_2);
+    BCI(cvtl_ignore_std_width);
+    BCI(0);
+    BCI(WCVTP);
+  }
+
+  ins_len = (FT_UInt)(bufp - ins_buf);
 
   if (ins_len > sfnt->max_instructions)
-    sfnt->max_instructions = ins_len;
+    sfnt->max_instructions = (FT_UShort)ins_len;
 
   glyph->ins_buf = (FT_Byte*)realloc(ins_buf, ins_len);
   glyph->ins_len = ins_len;
