@@ -1,7 +1,7 @@
 /* tabytecode.h */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -293,7 +293,7 @@
 
 
 /* a simple macro to emit bytecode instructions */
-#define BCI(code) *(bufp++) = (code)
+#define BCI(code) *(bufp++) = (FT_Byte)(code)
 
 /* we increase the stack depth by this amount */
 #define ADDITIONAL_STACK_ELEMENTS 20
@@ -375,24 +375,30 @@
 #define bci_create_segments_composite_9 bci_create_segments_composite_8 + 1
 
 /* 41 */
-#define bci_align_point bci_create_segments_composite_9 + 1
+/* the next three entries must stay in this order */
+#define bci_deltap1 bci_create_segments_composite_9 + 1
+#define bci_deltap2 bci_deltap1 + 1
+#define bci_deltap3 bci_deltap2 + 1
+
+/* 44 */
+#define bci_align_point bci_deltap3 + 1
 #define bci_align_segment bci_align_point + 1
 #define bci_align_segments bci_align_segment + 1
 
-/* 44 */
+/* 47 */
 #define bci_scale_contour bci_align_segments + 1
 #define bci_scale_glyph bci_scale_contour + 1
 #define bci_scale_composite_glyph bci_scale_glyph + 1
 #define bci_shift_contour bci_scale_composite_glyph + 1
 #define bci_shift_subglyph bci_shift_contour + 1
 
-/* 49 */
+/* 52 */
 #define bci_ip_outer_align_point bci_shift_subglyph + 1
 #define bci_ip_on_align_points bci_ip_outer_align_point + 1
 #define bci_ip_between_align_point bci_ip_on_align_points + 1
 #define bci_ip_between_align_points bci_ip_between_align_point + 1
 
-/* 53 */
+/* 56 */
 #define bci_adjust_common bci_ip_between_align_points + 1
 #define bci_stem_common bci_adjust_common + 1
 #define bci_serif_common bci_stem_common + 1
@@ -400,12 +406,12 @@
 #define bci_serif_link1_common bci_serif_anchor_common + 1
 #define bci_serif_link2_common bci_serif_link1_common + 1
 
-/* 59 */
+/* 62 */
 #define bci_lower_bound bci_serif_link2_common + 1
 #define bci_upper_bound bci_lower_bound + 1
 #define bci_upper_lower_bound bci_upper_bound + 1
 
-/* 62 */
+/* 65 */
 #define bci_adjust_bound bci_upper_lower_bound + 1
 #define bci_stem_bound bci_adjust_bound + 1
 #define bci_link bci_stem_bound + 1
@@ -416,23 +422,23 @@
 /* the order of the `bci_action_*' entries must correspond */
 /* to the order of the TA_Action enumeration entries (in `tahints.h') */
 
-/* 68 */
+/* 71 */
 #define bci_action_ip_before bci_stem + 1
 #define bci_action_ip_after bci_action_ip_before + 1
 #define bci_action_ip_on bci_action_ip_after + 1
 #define bci_action_ip_between bci_action_ip_on + 1
 
-/* 72 */
+/* 75 */
 #define bci_action_blue bci_action_ip_between + 1
 #define bci_action_blue_anchor bci_action_blue + 1
 
-/* 74 */
+/* 77 */
 #define bci_action_anchor bci_action_blue_anchor + 1
 #define bci_action_anchor_serif bci_action_anchor + 1
 #define bci_action_anchor_round bci_action_anchor_serif + 1
 #define bci_action_anchor_round_serif bci_action_anchor_round + 1
 
-/* 78 */
+/* 81 */
 #define bci_action_adjust bci_action_anchor_round_serif + 1
 #define bci_action_adjust_serif bci_action_adjust + 1
 #define bci_action_adjust_round bci_action_adjust_serif + 1
@@ -442,13 +448,13 @@
 #define bci_action_adjust_bound_round bci_action_adjust_bound_serif + 1
 #define bci_action_adjust_bound_round_serif bci_action_adjust_bound_round + 1
 
-/* 86 */
+/* 89 */
 #define bci_action_link bci_action_adjust_bound_round_serif + 1
 #define bci_action_link_serif bci_action_link + 1
 #define bci_action_link_round bci_action_link_serif + 1
 #define bci_action_link_round_serif bci_action_link_round + 1
 
-/* 90 */
+/* 93 */
 #define bci_action_stem bci_action_link_round_serif + 1
 #define bci_action_stem_serif bci_action_stem + 1
 #define bci_action_stem_round bci_action_stem_serif + 1
@@ -458,34 +464,37 @@
 #define bci_action_stem_bound_round bci_action_stem_bound_serif + 1
 #define bci_action_stem_bound_round_serif bci_action_stem_bound_round + 1
 
-/* 98 */
+/* 101 */
 #define bci_action_serif bci_action_stem_bound_round_serif + 1
 #define bci_action_serif_lower_bound bci_action_serif + 1
 #define bci_action_serif_upper_bound bci_action_serif_lower_bound + 1
 #define bci_action_serif_upper_lower_bound bci_action_serif_upper_bound + 1
 
-/* 102 */
+/* 105 */
 #define bci_action_serif_anchor bci_action_serif_upper_lower_bound + 1
 #define bci_action_serif_anchor_lower_bound bci_action_serif_anchor + 1
 #define bci_action_serif_anchor_upper_bound bci_action_serif_anchor_lower_bound + 1
 #define bci_action_serif_anchor_upper_lower_bound bci_action_serif_anchor_upper_bound + 1
 
-/* 106 */
+/* 109 */
 #define bci_action_serif_link1 bci_action_serif_anchor_upper_lower_bound + 1
 #define bci_action_serif_link1_lower_bound bci_action_serif_link1 + 1
 #define bci_action_serif_link1_upper_bound bci_action_serif_link1_lower_bound + 1
 #define bci_action_serif_link1_upper_lower_bound bci_action_serif_link1_upper_bound + 1
 
-/* 110 */
+/* 113 */
 #define bci_action_serif_link2 bci_action_serif_link1_upper_lower_bound + 1
 #define bci_action_serif_link2_lower_bound bci_action_serif_link2 + 1
 #define bci_action_serif_link2_upper_bound bci_action_serif_link2_lower_bound + 1
 #define bci_action_serif_link2_upper_lower_bound bci_action_serif_link2_upper_bound + 1
 
-/* 114 */
+/* 117 */
 #define bci_hint_glyph bci_action_serif_link2_upper_lower_bound + 1
 
-#define NUM_FDEFS bci_hint_glyph + 1 /* must be last */
+/* 118 */
+#define bci_freetype_enable_deltas bci_hint_glyph + 1
+
+#define NUM_FDEFS bci_freetype_enable_deltas + 1 /* must be last */
 
 /* the first action handler */
 #define ACTION_OFFSET bci_action_ip_before
@@ -499,8 +508,10 @@
 #define cvtl_is_subglyph cvtl_funits_to_pixels + 1
 #define cvtl_use_strong_functions cvtl_is_subglyph + 1
 #define cvtl_is_element cvtl_use_strong_functions + 1
+#define cvtl_do_iup_y cvtl_is_element + 1
+#define cvtl_ignore_std_width cvtl_do_iup_y + 1
 
-#define cvtl_max_runtime cvtl_is_element + 1 /* must be last */
+#define cvtl_max_runtime cvtl_ignore_std_width + 1 /* must be last */
 
 
 /* symbolic names for build-time CVT locations */
@@ -600,6 +611,7 @@
 
 
 extern FT_Byte ttfautohint_glyph_bytecode[7];
+extern FT_Byte ins_extra_buf[256];
 
 #endif /* __TABYTECODE_H__ */
 
