@@ -1,7 +1,7 @@
 /* taglyf.c */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -20,13 +20,21 @@ static FT_Error
 TA_sfnt_build_glyf_hints(SFNT* sfnt,
                          FONT* font)
 {
-  FT_Face face = sfnt->face;
+  SFNT_Table* glyf_table = &font->tables[sfnt->glyf_idx];
+  glyf_Data* data = (glyf_Data*)glyf_table->data;
+
   FT_Long idx;
   FT_Error error;
 
+  FT_UShort loop_count;
+
 
   /* this loop doesn't include the artificial `.ttfautohint' glyph */
-  for (idx = 0; idx < face->num_glyphs; idx++)
+  loop_count = data->num_glyphs;
+  if (sfnt->max_components && font->hint_composites)
+    loop_count--;
+
+  for (idx = 0; idx < loop_count; idx++)
   {
     error = TA_sfnt_build_glyph_instructions(sfnt, font, idx);
     if (error)
@@ -36,7 +44,7 @@ TA_sfnt_build_glyf_hints(SFNT* sfnt,
       FT_Int ret;
 
 
-      ret = font->progress(idx, face->num_glyphs,
+      ret = font->progress(idx, loop_count,
                            sfnt - font->sfnts, font->num_sfnts,
                            font->progress_data);
       if (ret)
@@ -73,12 +81,10 @@ TA_glyph_get_components(GLYPH* glyph,
     if (p + 4 > endp)
       return FT_Err_Invalid_Table;
 
-    flags = *(p++) << 8;
-    flags += *(p++);
+    flags = NEXT_USHORT(p);
 
     /* add component to list */
-    component = *(p++) << 8;
-    component += *(p++);
+    component = NEXT_USHORT(p);
 
     glyph->num_components++;
     components_new = (FT_UShort*)realloc(glyph->components,
@@ -188,8 +194,8 @@ TA_glyph_parse_composite(GLYPH* glyph,
       *(q++) = ARGS_ARE_XY_VALUES | MORE_COMPONENTS;
       *(q++) = HIGH(num_glyphs - 1);
       *(q++) = LOW(num_glyphs - 1);
-      *(q++) = x_offset;
-      *(q++) = y_offset;
+      *(q++) = (FT_Byte)x_offset;
+      *(q++) = (FT_Byte)y_offset;
     }
     else
     {
@@ -207,12 +213,11 @@ TA_glyph_parse_composite(GLYPH* glyph,
   /* walk over component records */
   do
   {
-    flags_offset = q - glyph->buf;
+    flags_offset = (FT_ULong)(q - glyph->buf);
 
     *(q++) = *p;
-    flags = *(p++) << 8;
-    *(q++) = *p;
-    flags += *(p++);
+    *(q++) = *(p + 1);
+    flags = NEXT_USHORT(p);
 
     /* copy component */
     *(q++) = *(p++);
@@ -267,8 +272,8 @@ TA_glyph_parse_composite(GLYPH* glyph,
       {
         glyph->buf[flags_offset + 1] &= ~ARGS_ARE_WORDS;
 
-        *(q++) = arg1;
-        *(q++) = arg2;
+        *(q++) = (FT_Byte)arg1;
+        *(q++) = (FT_Byte)arg2;
       }
       else
       {
@@ -301,7 +306,7 @@ TA_glyph_parse_composite(GLYPH* glyph,
     }
   } while (flags & MORE_COMPONENTS);
 
-  glyph->len1 = q - glyph->buf;
+  glyph->len1 = (FT_ULong)(q - glyph->buf);
   /* glyph->len2 = 0; */
   glyph->flags_offset = flags_offset;
   glyph->buf = (FT_Byte*)realloc(glyph->buf, glyph->len1);
@@ -335,7 +340,7 @@ TA_glyph_parse_simple(GLYPH* glyph,
   p = buf;
   endp = buf + len;
 
-  ins_offset = 10 + glyph->num_contours * 2;
+  ins_offset = 10 + (FT_ULong)glyph->num_contours * 2;
 
   p += ins_offset;
 
@@ -343,8 +348,7 @@ TA_glyph_parse_simple(GLYPH* glyph,
     return FT_Err_Invalid_Table;
 
   /* get number of instructions */
-  num_ins = *(p++) << 8;
-  num_ins += *(p++);
+  num_ins = NEXT_USHORT(p);
 
   /* assure that we don't process a font */
   /* which already contains a `.ttfautohint' glyph */
@@ -410,7 +414,7 @@ TA_glyph_parse_simple(GLYPH* glyph,
   if (p + xy_size > endp)
     return FT_Err_Invalid_Table;
 
-  flags_size = p - flags_start;
+  flags_size = (FT_ULong)(p - flags_start);
 
   /* store the data before and after the bytecode instructions */
   /* in the same array */
@@ -591,8 +595,8 @@ TA_sfnt_split_glyf_table(SFNT* sfnt,
 
   loca_format = head_table->buf[LOCA_FORMAT_OFFSET];
 
-  data->num_glyphs = loca_format ? loca_table->len / 4
-                                 : loca_table->len / 2;
+  data->num_glyphs = (FT_UShort)(loca_format ? loca_table->len / 4
+                                             : loca_table->len / 2);
   loop_count = data->num_glyphs - 1;
 
   /* allocate one more glyph slot if we have composite glyphs */
@@ -612,16 +616,10 @@ TA_sfnt_split_glyf_table(SFNT* sfnt,
   p = loca_table->buf;
 
   if (loca_format)
-  {
-    offset_next = *(p++) << 24;
-    offset_next += *(p++) << 16;
-    offset_next += *(p++) << 8;
-    offset_next += *(p++);
-  }
+    offset_next = NEXT_ULONG(p);
   else
   {
-    offset_next = *(p++) << 8;
-    offset_next += *(p++);
+    offset_next = NEXT_USHORT(p);
     offset_next <<= 1;
   }
 
@@ -634,16 +632,10 @@ TA_sfnt_split_glyf_table(SFNT* sfnt,
     offset = offset_next;
 
     if (loca_format)
-    {
-      offset_next = *(p++) << 24;
-      offset_next += *(p++) << 16;
-      offset_next += *(p++) << 8;
-      offset_next += *(p++);
-    }
+      offset_next = NEXT_ULONG(p);
     else
     {
-      offset_next = *(p++) << 8;
-      offset_next += *(p++);
+      offset_next = NEXT_USHORT(p);
       offset_next <<= 1;
     }
 
@@ -680,12 +672,11 @@ TA_sfnt_split_glyf_table(SFNT* sfnt,
 
 
         /* use the last contour's end point to compute number of points */
-        off = 10 + (glyph->num_contours - 1) * 2;
+        off = 10 + ((FT_ULong)glyph->num_contours - 1) * 2;
         if (off >= len - 1)
           return FT_Err_Invalid_Table;
 
-        glyph->num_points = buf[off] << 8;
-        glyph->num_points += buf[off + 1] + 1;
+        glyph->num_points = (FT_UShort)((buf[off] << 8) + buf[off + 1] + 1);
       }
     }
   }
@@ -702,16 +693,10 @@ TA_sfnt_split_glyf_table(SFNT* sfnt,
   p = loca_table->buf;
 
   if (loca_format)
-  {
-    offset_next = *(p++) << 24;
-    offset_next += *(p++) << 16;
-    offset_next += *(p++) << 8;
-    offset_next += *(p++);
-  }
+    offset_next = NEXT_ULONG(p);
   else
   {
-    offset_next = *(p++) << 8;
-    offset_next += *(p++);
+    offset_next = NEXT_USHORT(p);
     offset_next <<= 1;
   }
 
@@ -724,16 +709,10 @@ TA_sfnt_split_glyf_table(SFNT* sfnt,
     offset = offset_next;
 
     if (loca_format)
-    {
-      offset_next = *(p++) << 24;
-      offset_next += *(p++) << 16;
-      offset_next += *(p++) << 8;
-      offset_next += *(p++);
-    }
+      offset_next = NEXT_ULONG(p);
     else
     {
-      offset_next = *(p++) << 8;
-      offset_next += *(p++);
+      offset_next = NEXT_USHORT(p);
       offset_next <<= 1;
     }
 
@@ -843,8 +822,9 @@ TA_sfnt_build_glyf_table(SFNT* sfnt,
   for (i = 0; i < data->num_glyphs; i++, glyph++)
   {
     /* glyph records should have offsets which are multiples of 4 */
-    len = (len + 3) & ~3;
-    len += glyph->len1 + glyph->len2 + glyph->ins_len;
+    len = (len + 3) & ~3U;
+    len += glyph->len1 + glyph->len2
+           + glyph->ins_extra_len + glyph->ins_len;
     /* add two bytes for the instructionLength field */
     if (glyph->len2 || glyph->ins_len)
       len += 2;
@@ -852,9 +832,9 @@ TA_sfnt_build_glyf_table(SFNT* sfnt,
 
   /* to make the short format of the `loca' table always work, */
   /* assure an even length of the `glyf' table */
-  glyf_table->len = (len + 1) & ~1;
+  glyf_table->len = (len + 1) & ~1U;
 
-  buf_new = (FT_Byte*)realloc(glyf_table->buf, (len + 3) & ~3);
+  buf_new = (FT_Byte*)realloc(glyf_table->buf, (len + 3) & ~3U);
   if (!buf_new)
     return FT_Err_Out_Of_Memory;
   else
@@ -864,7 +844,8 @@ TA_sfnt_build_glyf_table(SFNT* sfnt,
   glyph = data->glyphs;
   for (i = 0; i < data->num_glyphs; i++, glyph++)
   {
-    len = glyph->len1 + glyph->len2 + glyph->ins_len;
+    len = glyph->len1 + glyph->len2
+          + glyph->ins_extra_len + glyph->ins_len;
     if (glyph->len2 || glyph->ins_len)
       len += 2;
 
@@ -877,8 +858,13 @@ TA_sfnt_build_glyf_table(SFNT* sfnt,
       {
         /* simple glyph */
         p += glyph->len1;
-        *(p++) = HIGH(glyph->ins_len);
-        *(p++) = LOW(glyph->ins_len);
+        *(p++) = HIGH(glyph->ins_extra_len + glyph->ins_len);
+        *(p++) = LOW(glyph->ins_extra_len + glyph->ins_len);
+        if (glyph->ins_extra_len)
+        {
+          memcpy(p, ins_extra_buf, glyph->ins_extra_len);
+          p += glyph->ins_extra_len;
+        }
         memcpy(p, glyph->ins_buf, glyph->ins_len);
         p += glyph->ins_len;
         memcpy(p, glyph->buf + glyph->len1, glyph->len2);
@@ -891,8 +877,13 @@ TA_sfnt_build_glyf_table(SFNT* sfnt,
         {
           *(p + glyph->flags_offset) |= (WE_HAVE_INSTR >> 8);
           p += glyph->len1;
-          *(p++) = HIGH(glyph->ins_len);
-          *(p++) = LOW(glyph->ins_len);
+          *(p++) = HIGH(glyph->ins_extra_len + glyph->ins_len);
+          *(p++) = LOW(glyph->ins_extra_len + glyph->ins_len);
+          if (glyph->ins_extra_len)
+          {
+            memcpy(p, ins_extra_buf, glyph->ins_extra_len);
+            p += glyph->ins_extra_len;
+          }
           memcpy(p, glyph->ins_buf, glyph->ins_len);
           p += glyph->ins_len;
         }
@@ -960,7 +951,7 @@ TA_create_glyph_data(FT_Outline* outline,
   /* we use `calloc' since we rely on the array */
   /* being initialized to zero; */
   /* additionally, we need one more byte for a test after the loop */
-  flags = (FT_Byte*)calloc(1, outline->n_points + 1);
+  flags = (FT_Byte*)calloc(1, (size_t)outline->n_points + 1);
   if (!flags)
   {
     error = FT_Err_Out_Of_Memory;
@@ -968,14 +959,14 @@ TA_create_glyph_data(FT_Outline* outline,
   }
 
   /* we have either one-byte or two-byte elements */
-  x = (FT_Byte*)malloc(2 * outline->n_points);
+  x = (FT_Byte*)malloc(2 * (size_t)outline->n_points);
   if (!x)
   {
     error = FT_Err_Out_Of_Memory;
     goto Exit;
   }
 
-  y = (FT_Byte*)malloc(2 * outline->n_points);
+  y = (FT_Byte*)malloc(2 * (size_t)outline->n_points);
   if (!y)
   {
     error = FT_Err_Out_Of_Memory;
@@ -1107,8 +1098,8 @@ TA_create_glyph_data(FT_Outline* outline,
 
   /* concatenate all arrays and fill needed GLYPH structure elements */
 
-  glyph->len1 = 10 + 2 * outline->n_contours;
-  glyph->len2 = (flagsp - flags) + (xp - x) + (yp - y);
+  glyph->len1 = (FT_ULong)(10 + 2 * outline->n_contours);
+  glyph->len2 = (FT_ULong)((flagsp - flags) + (xp - x) + (yp - y));
 
   glyph->buf = (FT_Byte*)malloc(glyph->len1 + glyph->len2);
   if (!glyph->buf)
@@ -1121,6 +1112,7 @@ TA_create_glyph_data(FT_Outline* outline,
   memcpy(p, header, 10);
   p += 10;
 
+  glyph->ins_extra_len = 0;
   glyph->ins_len = 0;
   glyph->ins_buf = NULL;
 
@@ -1130,11 +1122,11 @@ TA_create_glyph_data(FT_Outline* outline,
     *(p++) = LOW(outline->contours[i]);
   }
 
-  memcpy(p, flags, flagsp - flags);
+  memcpy(p, flags, (size_t)(flagsp - flags));
   p += flagsp - flags;
-  memcpy(p, x, xp - x);
+  memcpy(p, x, (size_t)(xp - x));
   p += xp - x;
-  memcpy(p, y, yp - y);
+  memcpy(p, y, (size_t)(yp - y));
 
 Exit:
   free(flags);
@@ -1176,7 +1168,7 @@ TA_sfnt_create_glyf_data(SFNT* sfnt,
 
   glyf_table->data = data;
 
-  data->num_glyphs = face->num_glyphs;
+  data->num_glyphs = (FT_UShort)face->num_glyphs;
   data->glyphs = (GLYPH*)calloc(1, data->num_glyphs * sizeof (GLYPH));
   if (!data->glyphs)
     return FT_Err_Out_Of_Memory;
@@ -1261,10 +1253,10 @@ TA_sfnt_handle_coverage(SFNT* sfnt,
     TA_FaceGlobals master_globals = data->master_globals;
     FT_Long count = master_globals->glyph_count;
 
-    FT_Byte* master = master_globals->glyph_styles;
-    FT_Byte* curr = curr_globals->glyph_styles;
+    FT_UShort* master = master_globals->glyph_styles;
+    FT_UShort* curr = curr_globals->glyph_styles;
 
-    FT_Byte* limit = master + count;
+    FT_UShort* limit = master + count;
 
 
     /* we simply copy the data, */
@@ -1274,7 +1266,7 @@ TA_sfnt_handle_coverage(SFNT* sfnt,
     /* for example, glyph `A' can be used for both Cyrillic and Latin */
     while (master < limit)
     {
-      if ((*curr & ~TA_DIGIT) != TA_STYLE_UNASSIGNED)
+      if ((*curr & TA_STYLE_MASK) != TA_STYLE_UNASSIGNED)
         *master = *curr;
 
       master++;
@@ -1301,7 +1293,7 @@ TA_sfnt_adjust_coverage(SFNT* sfnt,
   if (!data->adjusted)
   {
     FT_Long nn;
-    FT_Byte* gstyles = master_globals->glyph_styles;
+    FT_UShort* gstyles = master_globals->glyph_styles;
 #ifdef TA_DEBUG
     FT_UInt count;
 #endif
@@ -1309,9 +1301,9 @@ TA_sfnt_adjust_coverage(SFNT* sfnt,
 
     for (nn = 0; nn < master_globals->glyph_count; nn++)
     {
-      if ((gstyles[nn] & ~TA_DIGIT) == TA_STYLE_UNASSIGNED)
+      if ((gstyles[nn] & TA_STYLE_MASK) == TA_STYLE_UNASSIGNED)
       {
-        gstyles[nn] &= ~TA_STYLE_UNASSIGNED;
+        gstyles[nn] &= ~TA_STYLE_MASK;
         gstyles[nn] |= master_globals->font->fallback_style;
       }
     }
@@ -1333,7 +1325,8 @@ TA_sfnt_adjust_coverage(SFNT* sfnt,
 
     for (nn = 0; nn < master_globals->glyph_count; nn++)
     {
-      if ((gstyles[nn] & ~TA_DIGIT) == master_globals->font->fallback_style)
+      if ((gstyles[nn] & TA_STYLE_MASK)
+          == master_globals->font->fallback_style)
       {
         if (!(count % 10))
           TA_LOG_GLOBAL((" "));
@@ -1376,11 +1369,11 @@ TA_sfnt_copy_master_coverage(SFNT* sfnt,
   if (master_globals != curr_globals)
   {
     FT_Long count = master_globals->glyph_count;
-    FT_Byte* master = master_globals->glyph_styles;
-    FT_Byte* curr = curr_globals->glyph_styles;
+    FT_UShort* master = master_globals->glyph_styles;
+    FT_UShort* curr = curr_globals->glyph_styles;
 
 
-    memcpy(curr, master, count);
+    memcpy(curr, master, count * sizeof (FT_UShort));
   }
 }
 
