@@ -19,11 +19,8 @@
 /* the code below contains many redundancies; */
 /* it has been written for clarity */
 
-#define VALUE(val, p) val = *(p++) << 8; \
-                      val += *(p++)
-#define OFFSET(val, base, p) val = base; \
-                             val += *(p++) << 8; \
-                             val += *(p++)
+#define VALUE(val, p) val = NEXT_USHORT(p)
+#define OFFSET(val, base, p) val = base + NEXT_USHORT(p)
 
 
 /* this simple `Coverage_table' structure wastes memory... */
@@ -254,7 +251,6 @@ TA_handle_cursive_lookup(FT_Byte* Lookup,
       FT_UShort glyph_idx = cov.glyph_idxs[i];
       FT_Byte* EntryAnchor;
       FT_Byte* ExitAnchor;
-      FT_Error error;
 
 
       OFFSET(EntryAnchor, CursivePosFormat1, q);
@@ -339,7 +335,6 @@ TA_handle_markbase_lookup(FT_Byte* Lookup,
     {
       FT_UShort glyph_idx = cov.glyph_idxs[i];
       FT_Byte* MarkAnchor;
-      FT_Error error;
 
 
       q += 2; /* skip Class */
@@ -375,7 +370,6 @@ TA_handle_markbase_lookup(FT_Byte* Lookup,
       for (; cc > 0; cc--)
       {
         FT_Byte* BaseAnchor;
-        FT_Error error;
 
 
         OFFSET(BaseAnchor, BaseArray, q);
@@ -456,7 +450,6 @@ TA_handle_marklig_lookup(FT_Byte* Lookup,
     {
       FT_UShort glyph_idx = cov.glyph_idxs[i];
       FT_Byte* MarkAnchor;
-      FT_Error error;
 
 
       q += 2; /* skip Class */
@@ -505,7 +498,6 @@ TA_handle_marklig_lookup(FT_Byte* Lookup,
         for (; cc > 0; cc--)
         {
           FT_Byte* LigatureAnchor;
-          FT_Error error;
 
 
           OFFSET(LigatureAnchor, LigatureAttach, r);
@@ -587,7 +579,6 @@ TA_handle_markmark_lookup(FT_Byte* Lookup,
     {
       FT_UShort glyph_idx = cov.glyph_idxs[i];
       FT_Byte* Mark1Anchor;
-      FT_Error error;
 
 
       q += 2; /* skip Class */
@@ -623,7 +614,6 @@ TA_handle_markmark_lookup(FT_Byte* Lookup,
       for (; cc > 0; cc--)
       {
         FT_Byte* Mark2Anchor;
-        FT_Error error;
 
 
         OFFSET(Mark2Anchor, Mark2Array, q);
