@@ -1,7 +1,7 @@
 /* tahints.c */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -34,7 +34,15 @@ ta_axis_hints_new_segment(TA_AxisHints axis,
   TA_Segment segment = NULL;
 
 
-  if (axis->num_segments >= axis->max_segments)
+  if (axis->num_segments < TA_SEGMENTS_EMBEDDED)
+  {
+    if (axis->segments == NULL)
+    {
+      axis->segments = axis->embedded.segments;
+      axis->max_segments = TA_SEGMENTS_EMBEDDED;
+    }
+  }
+  else if (axis->num_segments >= axis->max_segments)
   {
     TA_Segment segments_new;
 
@@ -54,12 +62,26 @@ ta_axis_hints_new_segment(TA_AxisHints axis,
         || new_max > big_max)
       new_max = big_max;
 
-    segments_new = (TA_Segment)realloc(axis->segments,
-                                       new_max * sizeof (TA_SegmentRec));
-    if (!segments_new)
-      return FT_Err_Out_Of_Memory;
+    if (axis->segments == axis->embedded.segments)
+    {
+      axis->segments = (TA_Segment)malloc(
+                         (size_t)new_max * sizeof (TA_SegmentRec));
+      if (!axis->segments)
+        return FT_Err_Out_Of_Memory;
+
+      memcpy(axis->segments, axis->embedded.segments,
+             sizeof (axis->embedded.segments));
+    }
+    else
+    {
+      segments_new = (TA_Segment)realloc(
+                       axis->segments,
+                       (size_t)new_max * sizeof (TA_SegmentRec));
+      if (!segments_new)
+        return FT_Err_Out_Of_Memory;
+      axis->segments = segments_new;
+    }
 
-    axis->segments = segments_new;
     axis->max_segments = new_max;
   }
 
@@ -85,7 +107,15 @@ ta_axis_hints_new_edge(TA_AxisHints axis,
   TA_Edge edges;
 
 
-  if (axis->num_edges >= axis->max_edges)
+  if (axis->num_edges < TA_EDGES_EMBEDDED)
+  {
+    if (axis->edges == NULL)
+    {
+      axis->edges = axis->embedded.edges;
+      axis->max_edges = TA_EDGES_EMBEDDED;
+    }
+  }
+  else if (axis->num_edges >= axis->max_edges)
   {
     TA_Edge edges_new;
 
@@ -105,12 +135,24 @@ ta_axis_hints_new_edge(TA_AxisHints axis,
         || new_max > big_max)
       new_max = big_max;
 
-    edges_new = (TA_Edge)realloc(axis->edges,
-                                 new_max * sizeof (TA_EdgeRec));
-    if (!edges_new)
-      return FT_Err_Out_Of_Memory;
+    if (axis->edges == axis->embedded.edges)
+    {
+      axis->edges = (TA_Edge)malloc((size_t)new_max * sizeof (TA_EdgeRec));
+      if (!axis->edges)
+        return FT_Err_Out_Of_Memory;
+
+      memcpy(axis->edges, axis->embedded.edges,
+             sizeof (axis->embedded.edges));
+    }
+    else
+    {
+      edges_new = (TA_Edge)realloc(axis->edges,
+                                   (size_t)new_max * sizeof (TA_EdgeRec));
+      if (!edges_new)
+        return FT_Err_Out_Of_Memory;
+      axis->edges = edges_new;
+    }
 
-    axis->edges = edges_new;
     axis->max_edges = new_max;
   }
 
@@ -189,8 +231,8 @@ ta_dir_str(TA_Direction dir)
 
 
 #define TA_INDEX_NUM(ptr, base) \
-          ((ptr) ? ((ptr) - (base)) \
-                 : -1)
+          (int)((ptr) ? ((ptr) - (base)) \
+                      : -1)
 
 
 void
@@ -201,14 +243,18 @@ ta_glyph_hints_dump_points(TA_GlyphHints hints)
   TA_Point point;
 
 
-  TA_LOG(("Table of points:\n"
-          "  [ index |  xorg |  yorg | xscale | yscale"
-          " |  xfit |  yfit | flags ]\n"));
+  TA_LOG(("Table of points:\n"));
+
+  if (hints->num_points)
+    TA_LOG(("  [ index |  xorg |  yorg | xscale | yscale"
+            " |  xfit |  yfit | flags ]\n"));
+  else
+    TA_LOG(("  (none)\n"));
 
   for (point = points; point < limit; point++)
     TA_LOG(("  [ %5d | %5d | %5d | %6.2f | %6.2f"
             " | %5.2f | %5.2f |   %c   ]\n",
-            point - points,
+            TA_INDEX_NUM(point, points),
             point->fx,
             point->fy,
             point->ox / 64.0,
@@ -282,7 +328,7 @@ ta_glyph_hints_dump_segments(TA_GlyphHints hints)
       TA_LOG(("  [ %5d | %5.2g | %5s | %4d"
               " | %4d | %4d | %5d | %4d"
               " | %6d | %5d | %11s ]\n",
-              seg - segments,
+              TA_INDEX_NUM(seg, segments),
               dimension == TA_DIMENSION_HORZ ? (int)seg->first->ox / 64.0
                                              : (int)seg->first->oy / 64.0,
               ta_dir_str((TA_Direction)seg->dir),
@@ -331,7 +377,7 @@ ta_glyph_hints_dump_edges(TA_GlyphHints hints)
     for (edge = edges; edge < limit; edge++)
       TA_LOG(("  [ %5d | %5.2g | %5s | %4d"
               " | %5d |   %c  | %5.2f | %5.2f | %11s ]\n",
-              edge - edges,
+              TA_INDEX_NUM(edge, edges),
               (int)edge->opos / 64.0,
               ta_dir_str((TA_Direction)edge->dir),
               TA_INDEX_NUM(edge->link, edges),
@@ -383,15 +429,15 @@ ta_direction_compute(FT_Pos dx,
     else
     {
       dir = TA_DIR_DOWN;
-      ll = dy;
+      ll = -dy;
       ss = dx;
     }
   }
 
-  /* return no direction if arm lengths differ too much */
-  /* (value 14 is heuristic, corresponding to approx. 4.1 degrees) */
-  ss *= 14;
-  if (TA_ABS(ll) <= TA_ABS(ss))
+  /* return no direction if arm lengths do not differ enough */
+  /* (value 14 is heuristic, corresponding to approx. 4.1 degrees); */
+  /* the long arm is never negative */
+  if (ll <= 14 * TA_ABS(ss))
     dir = TA_DIR_NONE;
 
   return dir;
@@ -401,7 +447,8 @@ ta_direction_compute(FT_Pos dx,
 void
 ta_glyph_hints_init(TA_GlyphHints hints)
 {
-  memset(hints, 0, sizeof (TA_GlyphHintsRec));
+  /* no need to initialize the embedded items */
+  memset(hints, 0, sizeof (*hints) - sizeof (hints->embedded));
 }
 
 
@@ -423,24 +470,36 @@ ta_glyph_hints_done(TA_GlyphHints hints)
 
     axis->num_segments = 0;
     axis->max_segments = 0;
-    free(axis->segments);
-    axis->segments = NULL;
+    if (axis->segments != axis->embedded.segments)
+    {
+      free(axis->segments);
+      axis->segments = NULL;
+    }
 
     axis->num_edges = 0;
     axis->max_edges = 0;
-    free(axis->edges);
-    axis->edges = NULL;
+    if (axis->edges != axis->embedded.edges)
+    {
+      free(axis->edges);
+      axis->edges = NULL;
+    }
   }
 
-  free(hints->contours);
-  hints->contours = NULL;
+  if (hints->contours != hints->embedded.contours)
+  {
+    free(hints->contours);
+    hints->contours = NULL;
+  }
   hints->max_contours = 0;
   hints->num_contours = 0;
 
-  free(hints->points);
-  hints->points = NULL;
-  hints->num_points = 0;
+  if (hints->points != hints->embedded.points)
+  {
+    free(hints->points);
+    hints->points = NULL;
+  }
   hints->max_points = 0;
+  hints->num_points = 0;
 }
 
 
@@ -522,13 +581,25 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
 
   /* first of all, reallocate the contours array if necessary */
   new_max = (FT_UInt)outline->n_contours;
-  old_max = hints->max_contours;
-  if (new_max > old_max)
+  old_max = (FT_UInt)hints->max_contours;
+
+  if (new_max <= TA_CONTOURS_EMBEDDED)
+  {
+    if (hints->contours == NULL)
+    {
+      hints->contours = hints->embedded.contours;
+      hints->max_contours = TA_CONTOURS_EMBEDDED;
+    }
+  }
+  else if (new_max > old_max)
   {
     TA_Point* contours_new;
 
 
-    new_max = (new_max + 3) & ~3; /* round up to a multiple of 4 */
+    if (hints->contours == hints->embedded.contours)
+      hints->contours = NULL;
+
+    new_max = (new_max + 3) & ~3U; /* round up to a multiple of 4 */
 
     contours_new = (TA_Point*)realloc(hints->contours,
                                       new_max * sizeof (TA_Point));
@@ -536,19 +607,31 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
       return FT_Err_Out_Of_Memory;
 
     hints->contours = contours_new;
-    hints->max_contours = new_max;
+    hints->max_contours = (FT_Int)new_max;
   }
 
   /* reallocate the points arrays if necessary -- we reserve */
   /* two additional point positions, used to hint metrics appropriately */
   new_max = (FT_UInt)(outline->n_points + 2);
-  old_max = hints->max_points;
-  if (new_max > old_max)
+  old_max = (FT_UInt)hints->max_points;
+
+  if (new_max <= TA_POINTS_EMBEDDED)
+  {
+    if (hints->points == NULL)
+    {
+      hints->points = hints->embedded.points;
+      hints->max_points = TA_POINTS_EMBEDDED;
+    }
+  }
+  else if (new_max > old_max)
   {
     TA_Point points_new;
 
 
-    new_max = (new_max + 2 + 7) & ~7; /* round up to a multiple of 8 */
+    if (hints->points == hints->embedded.points)
+      hints->points = NULL;
+
+    new_max = (new_max + 2 + 7) & ~7U; /* round up to a multiple of 8 */
 
     points_new = (TA_Point)realloc(hints->points,
                                    new_max * sizeof (TA_PointRec));
@@ -556,7 +639,7 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
       return FT_Err_Out_Of_Memory;
 
     hints->points = points_new;
-    hints->max_points = new_max;
+    hints->max_points = (FT_Int)new_max;
   }
 
   hints->num_points = outline->n_points;
@@ -684,8 +767,6 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
 
         FT_Pos out_x, out_y;
 
-        FT_Bool is_first;
-
 
         /* since the first point of a contour could be part of a */
         /* series of near points, go backwards to find the first */
@@ -735,17 +816,13 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
         out_x = 0;
         out_y = 0;
 
-        is_first = 1;
-
-        for (point = first;
-             point != first || is_first;
-             point = point->next)
+        next = first;
+        do
         {
           TA_Direction out_dir;
 
 
-          is_first = 0;
-
+          point = next;
           next = point->next;
 
           out_x += next->fx - point->fx;
@@ -777,7 +854,8 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
 
           out_x = 0;
           out_y = 0;
-        }
+
+        } while (next != first);
       }
 
       /*
@@ -1249,33 +1327,27 @@ ta_iup_interp(TA_Point p1,
               TA_Point ref2)
 {
   TA_Point p;
-  FT_Pos u;
-  FT_Pos v1 = ref1->v;
-  FT_Pos v2 = ref2->v;
-  FT_Pos d1 = ref1->u - v1;
-  FT_Pos d2 = ref2->u - v2;
+  FT_Pos u, v1, v2, u1, u2, d1, d2;
 
 
   if (p1 > p2)
     return;
 
-  if (v1 == v2)
+  if (ref1->v > ref2->v)
   {
-    for (p = p1; p <= p2; p++)
-    {
-      u = p->v;
-
-      if (u <= v1)
-        u += d1;
-      else
-        u += d2;
-
-      p->u = u;
-    }
-    return;
+    p = ref1;
+    ref1 = ref2;
+    ref2 = p;
   }
 
-  if (v1 < v2)
+  v1 = ref1->v;
+  v2 = ref2->v;
+  u1 = ref1->u;
+  u2 = ref2->u;
+  d1 = u1 - v1;
+  d2 = u2 - v2;
+
+  if (u1 == u2 || v1 == v2)
   {
     for (p = p1; p <= p2; p++)
     {
@@ -1286,23 +1358,26 @@ ta_iup_interp(TA_Point p1,
       else if (u >= v2)
         u += d2;
       else
-        u = ref1->u + FT_MulDiv(u - v1, ref2->u - ref1->u, v2 - v1);
+        u = u1;
 
       p->u = u;
     }
   }
   else
   {
+    FT_Fixed scale = FT_DivFix(u2 - u1, v2 - v1);
+
+
     for (p = p1; p <= p2; p++)
     {
       u = p->v;
 
-      if (u <= v2)
-        u += d2;
-      else if (u >= v1)
+      if (u <= v1)
         u += d1;
+      else if (u >= v2)
+        u += d2;
       else
-        u = ref1->u + FT_MulDiv(u - v1, ref2->u - ref1->u, v2 - v1);
+        u = u1 + FT_MulFix(u - v1, scale);
 
       p->u = u;
     }
