@@ -1,7 +1,7 @@
 /* tahints.h */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -208,24 +208,24 @@ typedef enum TA_Direction_
 #define TA_FLAG_NONE 0
 
 /* point type flags */
-#define TA_FLAG_CONIC (1 << 0)
-#define TA_FLAG_CUBIC (1 << 1)
+#define TA_FLAG_CONIC (1U << 0)
+#define TA_FLAG_CUBIC (1U << 1)
 #define TA_FLAG_CONTROL (TA_FLAG_CONIC | TA_FLAG_CUBIC)
 
 /* point touch flags */
-#define TA_FLAG_TOUCH_X (1 << 2)
-#define TA_FLAG_TOUCH_Y (1 << 3)
+#define TA_FLAG_TOUCH_X (1U << 2)
+#define TA_FLAG_TOUCH_Y (1U << 3)
 
 /* candidates for weak interpolation have this flag set */
-#define TA_FLAG_WEAK_INTERPOLATION (1 << 4)
+#define TA_FLAG_WEAK_INTERPOLATION (1U << 4)
 
 
 /* edge hint flags */
 #define TA_EDGE_NORMAL 0
-#define TA_EDGE_ROUND (1 << 0)
-#define TA_EDGE_SERIF (1 << 1)
-#define TA_EDGE_DONE (1 << 2)
-#define TA_EDGE_NEUTRAL (1 << 3)
+#define TA_EDGE_ROUND (1U << 0)
+#define TA_EDGE_SERIF (1U << 1)
+#define TA_EDGE_DONE (1U << 2)
+#define TA_EDGE_NEUTRAL (1U << 3)
 
 
 typedef struct TA_PointRec_* TA_Point;
@@ -299,6 +299,9 @@ typedef struct TA_EdgeRec_
 } TA_EdgeRec;
 
 
+#define TA_SEGMENTS_EMBEDDED 18 /* number of embedded segments */
+#define TA_EDGES_EMBEDDED 12 /* number of embedded edges */
+
 typedef struct TA_AxisHintsRec_
 {
   FT_Int num_segments; /* number of used segments */
@@ -313,6 +316,13 @@ typedef struct TA_AxisHintsRec_
   TA_Edge edges; /* edges array */
 
   TA_Direction major_dir; /* either vertical or horizontal */
+
+  /* two arrays to avoid allocation penalty */
+  struct
+  {
+    TA_SegmentRec segments[TA_SEGMENTS_EMBEDDED];
+    TA_EdgeRec edges[TA_EDGES_EMBEDDED];
+  } embedded;
 } TA_AxisHintsRec, *TA_AxisHints;
 
 
@@ -392,6 +402,10 @@ typedef void
                      TA_Edge lower_bound,
                      TA_Edge upper_bound);
 
+
+#define TA_POINTS_EMBEDDED 96 /* number of embedded points */
+#define TA_CONTOURS_EMBEDDED 8 /* number of embedded contours */
+
 typedef struct TA_GlyphHintsRec_
 {
   FT_Fixed x_scale;
@@ -419,6 +433,14 @@ typedef struct TA_GlyphHintsRec_
 
   TA_Hints_Recorder recorder;
   void* user;
+
+  /* two arrays to avoid allocation penalty; */
+  /* the `embedded' structure must be the last element! */
+  struct
+  {
+    TA_Point contours[TA_CONTOURS_EMBEDDED];
+    TA_PointRec points[TA_POINTS_EMBEDDED];
+  } embedded;
 } TA_GlyphHintsRec;
 
 
@@ -438,9 +460,6 @@ typedef struct TA_GlyphHintsRec_
           (!_ta_debug_disable_vert_hints \
            && !TA_HINTS_TEST_SCALER(h, TA_SCALER_FLAG_NO_VERTICAL))
 
-#define TA_HINTS_DO_ADVANCE(h) \
-          !TA_HINTS_TEST_SCALER(h, TA_SCALER_FLAG_NO_ADVANCE)
-
 #define TA_HINTS_DO_BLUES(h) \
           (!_ta_debug_disable_blue_hints)
 
@@ -452,15 +471,19 @@ typedef struct TA_GlyphHintsRec_
 #define TA_HINTS_DO_VERTICAL(h) \
           !TA_HINTS_TEST_SCALER(h, TA_SCALER_FLAG_NO_VERTICAL)
 
-#define TA_HINTS_DO_ADVANCE(h) \
-          !TA_HINTS_TEST_SCALER(h, TA_SCALER_FLAG_NO_ADVANCE)
-
 #define TA_HINTS_DO_BLUES(h) \
           1
 
 #endif /* !TA_DEBUG */
 
 
+#define TA_HINTS_DO_ADVANCE(h) \
+          !TA_HINTS_TEST_SCALER(h, TA_SCALER_FLAG_NO_ADVANCE)
+
+#define TA_HINTS_DO_WARP(h) \
+          !TA_HINTS_TEST_SCALER(h, TA_SCALER_FLAG_NO_WARPER)
+
+
 TA_Direction
 ta_direction_compute(FT_Pos dx,
                      FT_Pos dy);
