@@ -101,7 +101,7 @@ ta_sort_and_quantize_widths(FT_UInt* count,
         sum += table[j].org;
         table[j].org = 0;
       }
-      table[cur_idx].org = sum / j;
+      table[cur_idx].org = sum / (FT_Pos)j;
 
       if (i < *count - 1)
       {
