@@ -1,7 +1,7 @@
 /* tasort.h */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -22,6 +22,10 @@
 
 #include "tatypes.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 
 void
 ta_sort_pos(FT_UInt count,
@@ -32,6 +36,10 @@ ta_sort_and_quantize_widths(FT_UInt* count,
                             TA_Width widths,
                             FT_Pos threshold);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* __TASORT_H__ */
 
 /* end of tasort.h */
