@@ -1,7 +1,7 @@
 /* tattfa.c */
 
 /*
- * Copyright (C) 2014 by Werner Lemberg.
+ * Copyright (C) 2014-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -35,7 +35,7 @@ TA_table_build_TTFA(FT_Byte** TTFA,
   buf_len = strlen((char*)buf);
 
   /* buffer length must be a multiple of four */
-  len = (buf_len + 3) & ~3;
+  len = (buf_len + 3) & ~3U;
   buf_new = (FT_Byte*)realloc(buf, len);
   if (!buf_new)
   {
