@@ -1,7 +1,7 @@
 /* ttfautohint.c */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -28,7 +28,7 @@
            && !strncmp(start, str, sizeof (str) - 1))
 
 
-void
+static void
 TA_sfnt_set_properties(SFNT* sfnt,
                        FONT* font)
 {
@@ -88,6 +88,7 @@ TTF_autohint(const char* options,
   TA_Error_Func err = NULL;
   void* err_data = NULL;
   TA_Info_Func info = NULL;
+  TA_Info_Post_Func info_post = NULL;
   void* info_data = NULL;
 
   FT_Bool windows_compatibility = 0;
@@ -138,7 +139,7 @@ TTF_autohint(const char* options,
     if (start == op)
       goto End;
 
-    len = op - start;
+    len = (size_t)(op - start);
 
     /* the `COMPARE' macro uses `len' and `start' */
 
@@ -215,6 +216,8 @@ TTF_autohint(const char* options,
       info = va_arg(ap, TA_Info_Func);
     else if (COMPARE("info-callback-data"))
       info_data = va_arg(ap, void*);
+    else if (COMPARE("info-post-callback"))
+      info_post = va_arg(ap, TA_Info_Post_Func);
     else if (COMPARE("out-buffer"))
     {
       out_file = NULL;
@@ -321,9 +324,6 @@ TTF_autohint(const char* options,
 
   if (fallback_script_string)
   {
-    int i;
-
-
     for (i = 0; i < TA_STYLE_MAX; i++)
     {
       TA_StyleClass style_class = ta_style_classes[i];
@@ -345,9 +345,6 @@ TTF_autohint(const char* options,
 
   if (default_script_string)
   {
-    int i;
-
-
     for (i = 0; i < TA_SCRIPT_MAX; i++)
     {
       if (!strcmp(script_names[i], default_script_string))
@@ -383,7 +380,7 @@ TTF_autohint(const char* options,
   font->hinting_range_min = (FT_UInt)hinting_range_min;
   font->hinting_range_max = (FT_UInt)hinting_range_max;
   font->hinting_limit = (FT_UInt)hinting_limit;
-  font->increase_x_height = increase_x_height;
+  font->increase_x_height = (FT_UInt)increase_x_height;
   font->x_height_snapping_exceptions = x_height_snapping_exceptions;
   font->fallback_stem_width = (FT_UInt)fallback_stem_width;
 
@@ -403,6 +400,7 @@ No_check:
   font->progress = progress;
   font->progress_data = progress_data;
   font->info = info;
+  font->info_post = info_post;
   font->info_data = info_data;
 
   font->debug = debug;
@@ -461,8 +459,11 @@ No_check:
     FT_UInt idx;
 
 
-    error = FT_New_Memory_Face(font->lib, font->in_buf, font->in_len,
-                               i, &sfnt->face);
+    error = FT_New_Memory_Face(font->lib,
+                               font->in_buf,
+                               (FT_Long)font->in_len,
+                               i,
+                               &sfnt->face);
 
     /* assure that the font hasn't been already processed by ttfautohint; */
     /* another, more thorough check is done in TA_glyph_parse_simple */
@@ -588,6 +589,17 @@ No_check:
   }
 #endif
 
+  if (!font->dehint)
+  {
+    for (i = 0; i < font->num_sfnts; i++)
+    {
+      SFNT* sfnt = &font->sfnts[i];
+
+
+      TA_control_apply_coverage(sfnt, font);
+    }
+  }
+
   /* loop again over subfonts */
   for (i = 0; i < font->num_sfnts; i++)
   {
