@@ -1,7 +1,7 @@
 /* ttfautohint.h */
 
 /*
- * Copyright (C) 2011-2014 by Werner Lemberg.
+ * Copyright (C) 2011-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -153,7 +153,12 @@ typedef void
  * Callback: `TA_Info_Func`
  * ------------------------
  *
- * A callback function to manipulate strings in the `name` table.
+ * A callback function to access or modify strings in the `name` table; it
+ * is called in a loop that iterates over all `name` table entries.  If
+ * defined, [`TA_Info_Post_Func`](#callback-ta_info_post_func) gets executed
+ * after this loop so that the collected data can be written back to the
+ * `name` table.
+ *
  * *platform_id*, *encoding_id*, *language_id*, and *name_id* are the
  * identifiers of a `name` table entry pointed to by *str* with a length
  * pointed to by *str_len* (in bytes; the string has no trailing NULL byte).
@@ -189,6 +194,40 @@ typedef int
  *
  */
 
+
+/*
+ * Callback: `TA_Info_Post_Func`
+ * -----------------------------
+ *
+ * A callback function, giving the application the possibility to access or
+ * modify strings in the `name` table after
+ * [`TA_Info_Func`](#callback-ta_info_func) has iterated over all `name`
+ * table entries.
+ *
+ * It is expected that `TA_Info_Func` stores pointers to the `name` table
+ * entries it wants to access or modify; the only parameter is thus
+ * *info_data*, which is a void pointer to the user-supplied data already
+ * provided to `TA_Info_Func`.  Obviously, calling `TA_Info_Post_Func` with
+ * `TA_Info_Func` undefined has no effect.
+ *
+ * The `name` table strings are allocated with `malloc`; the application
+ * should reallocate the data if necessary, ensuring that no string length
+ * exceeds 0xFFFF.
+ *
+ * If an error occurs, return a non-zero value and don't modify the affected
+ * string and string length (such errors are handled as non-fatal).
+ *
+ * ```C
+ */
+
+typedef int
+(*TA_Info_Post_Func)(void* info_data);
+
+/*
+ * ```
+ *
+ */
+
 /* pandoc-end */
 
 
@@ -253,74 +292,8 @@ typedef int
  * :   A pointer of type `FILE*` to the data stream of control instructions.
  *     Mutually exclusive with `control-buffer`.
  *
- *     An entry in a control instructions file or buffer has one of the
- *     following syntax forms:
- *
- *     > *\[* font-idx *\]*\ \ glyph-id\ \ *`l`|`r`* points\ \ *\[* *`(`* left-offset *`,`* right-offset *`)`* *\]*\
- *     > *\[* font-idx *\]*\ \ glyph-id\ \ *`n`* points\
- *     > *\[* font-idx *\]*\ \ glyph-id\ \ *`p`* points\ \ *\[* *`x`* x-shift *\]*\ \ *\[* *`y`* y-shift *\]*\ \ *`@`* ppems
- *
- *     *font-idx* gives the index of the font in a TrueType Collection.  If
- *     missing, it is set to zero.  For normal TrueType fonts, only value
- *     zero is valid.  If starting with `0x` the number is interpreted as
- *     hexadecimal.  If starting with `0` it gets interpreted as an octal
- *     value, and as a decimal value otherwise.
- *
- *     *glyph-id* is a glyph's name as listed in the `post` SFNT table or a
- *     glyph index.  A glyph name consists of characters from the set
- *     '`A-Za-z0-9._`' only and does not start with a digit or period, with
- *     the exceptions of the names '`.notdef`' and '`.null`'.  A glyph index
- *     can be specified in decimal, octal, or hexadecimal format, the latter
- *     two indicated by the prefixes `0` and `0x`, respectively.
- *
- *     The mutually exclusive parameters '`l`' and '`r`' indicate that the
- *     following points have left or right 'out' direction, respectively,
- *     overriding ttfautohint's algorithm for setting point directions.  The
- *     'out direction' of a point is the direction of the outline *leaving*
- *     the point (or passing the control point).  If the specified direction
- *     is identical to what ttfautohint computes, nothing special happens.
- *     Otherwise, a one-point segment with the specified direction gets
- *     created.  By default, its length is zero.  Setting *left-offset* and
- *     *right-offset*, you can change the segment's horizontal start and end
- *     position relative to the point position.  *left-offset* and
- *     *right-offset* are integers measured in font units.
- *
- *     Parameter '`n`' sets the 'out' direction of the following points to
- *     'no direction'.  If the specified direction is identical to what
- *     ttfautohint computes, nothing special happens.  Otherwise,
- *     ttfautohint no longer considers those points as part of horizontal
- *     segments, thus treating them as 'weak' points.
- *
- *     Modifying or adding segments don't directly modify the outlines; it
- *     only influences the hinting process.
- *
- *     Parameter '`p`' makes ttfautohint apply delta exceptions for the
- *     given points, shifting the points by the given values.  Note that
- *     those delta exceptions are applied *after* the final `IUP`
- *     instructions in the bytecode; as a consequence, they are (partially)
- *     ignored by rasterizers if in ClearType mode.
- *
- *     Both *points* and *ppems* are number ranges, similar to the
- *     `x-height-snapping-exceptions` syntax.
- *
- *     *x-shift* and *y-shift* represent real numbers that get rounded to
- *     multiples of 1/8 pixels.  The entries for '`x`' and '`y`' are
- *     optional; if missing, the corresponding value is set to zero.
- *
- *     Values for *x-shift* and *y-shift* must be in the range [-1.0;1.0].
- *     Values for *ppems* must be in the range [6;53].  Values for *points*
- *     are limited by the number of points in the glyph.
- *
- *     Control instruction entries can be either separated with newlines or
- *     with character '`;`'.  Additionally, a line can be continued on the
- *     next line by ending it with backslash character ('`\`').  A backslash
- *     followed by a newline gets treated similar to a whitespace character.
- *
- *     A comment starts with character '`#`'; the rest of the line is
- *     ignored.  An empty line is ignored also.
- *
- *     Note that only character '`.`' is recognized as a decimal point, and
- *     a thousands separator is not accepted.
+ *     See '[Control Instructions](#control-instructions)' for the syntax
+ *     used in such a file or buffer.
  *
  * `control-buffer`
  * :   A pointer of type `const char*` to a buffer that contains control
@@ -365,11 +338,18 @@ typedef int
  * :   A pointer of type [`TA_Info_Func`](#callback-ta_info_func),
  *     specifying a callback function for manipulating the `name` table.
  *     This function gets called for each `name` table entry.  If not set or
- *     set to NULL, the table data stays unmodified.
+ *     set to NULL, `TA_Info_Func` is not called.
+ *
+ * `info-post-callback`
+ * :   A pointer of type [`TA_Info_Post_Func`](#callback-ta_info_post_func),
+ *     specifying a callback function for manipulating the `name` table.  It
+ *     is called after the function specified with `info-callback` has
+ *     iterated over all `name` table entries.  If not set or set to NULL,
+ *     `TA_Info_Post_Func` is not called.
  *
  * `info-callback-data`
  * :   A pointer of type `void*` to user data that is passed to the info
- *     callback function.
+ *     callback functions.
  *
  * `debug`
  * :   If this integer is set to\ 1, lots of debugging information is print
